
local Graph = LibStub:GetLibrary("LibGraph-2.0")


local function AlarmHUDpie_ShortNumber(h)
  if h > 999999 then
    return format("%.1fm", h / 1000000)
  elseif h > 9999 then
    return format("%.1fk", h / 1000)
  else
    return format("%d", h)
  end
end

local pieRadius = 50
local pieOffsetX = UIParent:GetWidth() / 7
local pieOffsetY = UIParent:GetHeight() / 7

local targetPowerPie = Graph:CreateGraphPieChart("AlarmHUDpieTargetPower", UIParent, "CENTER", "CENTER", -pieOffsetX, pieOffsetY, pieRadius * 1.1, pieRadius * 1.1)

targetPowerPie:RegisterEvent("PLAYER_TARGET_CHANGED")
targetPowerPie:RegisterEvent("UNIT_DISPLAYPOWER")
targetPowerPie:RegisterEvent("UNIT_MANA")
targetPowerPie:RegisterEvent("UNIT_MAXMANA")

targetPowerPie:SetScript("OnEvent",
  function(frame, event, ...)
    if arg1 == "target" or event == "PLAYER_TARGET_CHANGED" then
      frame:ResetPie()

      if UnitExists("target") then
        local p = (100 * UnitPower("target")) / UnitPowerMax("target")
        local col = PowerBarColor[UnitPowerType("target")]

        if p >= 100 then
          frame:CompletePie({col.r, col.g, col.b})
        else
          frame:AddPie(p, {col.r, col.g, col.b})
          frame:CompletePie({0, 0, 0})
        end
      end
    end
  end
)


local targetPie = Graph:CreateGraphPieChart("AlarmHUDpieTarget", UIParent, "CENTER", "CENTER", -pieOffsetX, pieOffsetY, pieRadius, pieRadius)
targetPie:SetFrameLevel(targetPie:GetFrameLevel() + 1)

local targetNumber = UIParent:CreateFontString("AlarmHUDpieTargetNumber", "LOW", "GameFontNormalHuge")
targetNumber:SetPoint("LEFT", targetPowerPie, "RIGHT", 5, 0)

local targetName = UIParent:CreateFontString("AlarmHUDpieTargetName", "LOW", "GameFontNormal")
targetName:SetPoint("BOTTOM", targetPowerPie, "TOP", 0, 5)

local targetPercent = UIParent:CreateFontString("AlarmHUDpieTargetPercent", "LOW", "GameFontNormal")
targetPercent:SetPoint("TOP", targetPowerPie, "BOTTOM", 0, -5)

local targetIcon = UIParent:CreateTexture("AlarmHUDpieTargetIcon", "OVERLAY")
targetIcon:SetHeight(pieRadius * 0.75)
targetIcon:SetWidth(pieRadius * 0.75)
targetIcon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
targetIcon:SetPoint("RIGHT", targetPowerPie, "LEFT", -5, 0)
targetIcon:Hide()

targetPie:RegisterEvent("PLAYER_TARGET_CHANGED")
targetPie:RegisterEvent("RAID_TARGET_UPDATE")
targetPie:RegisterEvent("UNIT_HEALTH")
targetPie:RegisterEvent("UNIT_MAXHEALTH")

targetPie:SetScript("OnEvent",
  function(frame, event, ...)
    if arg1 == "target" or event == "PLAYER_TARGET_CHANGED" or event == "RAID_TARGET_UPDATE" then
      frame:ResetPie()

      if UnitExists("target") then
        local p = (100 * UnitHealth("target")) / UnitHealthMax("target")
        local _, c = UnitClass("target")

        if not c then
          c = "WARRIOR"
        end

        local col = RAID_CLASS_COLORS[c]
        local idx = GetRaidTargetIndex("target")

        if idx then
          SetRaidTargetIconTexture(targetIcon, idx)
          targetIcon:Show()
        else
          targetIcon:Hide()
        end

        if event == "PLAYER_TARGET_CHANGED" then
          targetName:SetText(UnitName("target"))
          targetName:SetTextColor(col.r, col.g, col.b)
        end

        targetNumber:SetText(AlarmHUDpie_ShortNumber(UnitHealth("target")))
        targetPercent:SetFormattedText("%d%%", (100 * UnitHealth("target")) / UnitHealthMax("target"))

        if p >= 100 then
          frame:AddPie(35, {0.7, 0.2, 0.2})
          frame:CompletePie({0.6, 0.2, 0.2})
        elseif p > 35 then
          frame:AddPie(35, {0.7, 0.2, 0.2})
          frame:AddPie(p - 35, {0.6, 0.2, 0.2})
          frame:CompletePie({0, 0, 0})
        else
          frame:AddPie(p, {1, 0.2, 0.2})
          frame:CompletePie({0, 0, 0})
        end
      else
        targetName:SetText("")
        targetNumber:SetText("")
        targetPercent:SetText("")
        targetIcon:Hide()
      end
    end
  end
)


local threatPie = Graph:CreateGraphPieChart("AlarmHUDpieThreat", UIParent, "CENTER", "CENTER", pieOffsetX, pieOffsetY, pieRadius, pieRadius)

local threatNumber = UIParent:CreateFontString("AlarmHUDpieThreatNumber", "LOW", "GameFontNormalHuge")
threatNumber:SetPoint("RIGHT", threatPie, "LEFT", -5, 0)

local threatStatus = UIParent:CreateFontString("AlarmHUDpieThreatStatus", "LOW", "GameFontNormal")
threatStatus:SetPoint("TOP", threatPie, "BOTTOM", 0, -5)

threatPie:RegisterEvent("PLAYER_TARGET_CHANGED")
threatPie:RegisterEvent("UNIT_THREAT_LIST_UPDATE")

threatPie:SetScript("OnEvent",
  function(frame, event, ...)
    if arg1 == "target" or event == "PLAYER_TARGET_CHANGED" then
      frame:ResetPie()

      if UnitExists("target") then
        local _, s, p, _, _ = UnitDetailedThreatSituation("player", "target")
        local r, g, b = GetThreatStatusColor(s)
        local ts = {"Low Threat", "HIGH THREAT", "Losing Threat", "TANKING"}
        
        if p == nil then
          threatStatus:SetText("No Threat")
          threatNumber:SetText("")
          frame:CompletePie({0, 0.3, 0})
        else
          threatStatus:SetText(ts[s + 1])
          threatNumber:SetFormattedText("%d%%", p)

          if p >= 100 then
            frame:CompletePie({r, g, b})
          else
            frame:AddPie(p, {r, g, b})
            frame:CompletePie({0, 0, 0})
          end
        end
      else
        threatStatus:SetText("")
        threatNumber:SetText("")
      end
    end
  end
)


local healthPie = Graph:CreateGraphPieChart("AlarmHUDpieHealth", UIParent, "CENTER", "CENTER", -pieOffsetX, -pieOffsetY, pieRadius, pieRadius)

local healthValue = UIParent:CreateFontString("AlarmHUDpieHealthValue", "LOW", "GameFontNormal")
healthValue:SetPoint("BOTTOM", healthPie, "TOP", 0, 5)

local healthMaxValue = UIParent:CreateFontString("AlarmHUDpieHealthMaxValue", "LOW", "GameFontNormalSmall")
healthMaxValue:SetPoint("TOP", healthPie, "BOTTOM", 0, -5)

healthPie:RegisterEvent("PLAYER_ENTERING_WORLD")
healthPie:RegisterEvent("UNIT_HEALTH")
healthPie:RegisterEvent("UNIT_MAXHEALTH")

healthPie:SetScript("OnEvent",
  function(frame, event, ...)
    if arg1 == "player" or event == "PLAYER_ENTERING_WORLD" then
      frame:ResetPie()

      if UnitExists("player") then
        local p = (100 * UnitHealth("player")) / UnitHealthMax("player")

        healthMaxValue:SetText(UnitHealthMax("player"))
        healthValue:SetText(UnitHealth("player"))

        if p >= 100 then
          frame:CompletePie({0, 1, 0})
        else
          frame:AddPie(p, {0, 1, 0})
          frame:CompletePie({0, 0, 0})
        end
      else
        healthMaxValue:SetText("")
        healthValue:SetText("")
      end
    end
  end
)


local powerPie = Graph:CreateGraphPieChart("AlarmHUDpiePower", UIParent, "CENTER", "CENTER", pieOffsetX, -pieOffsetY, pieRadius, pieRadius)

local powerValue = UIParent:CreateFontString("AlarmHUDpiePowerValue", "LOW", "GameFontNormal")
powerValue:SetPoint("BOTTOM", powerPie, "TOP", 0, 5)

local powerMaxValue = UIParent:CreateFontString("AlarmHUDpiePowerMaxValue", "LOW", "GameFontNormalSmall")
powerMaxValue:SetPoint("TOP", powerPie, "BOTTOM", 0, -5)

powerPie:RegisterEvent("PLAYER_ENTERING_WORLD")
powerPie:RegisterEvent("UNIT_DISPLAYPOWER")
if select(2, UnitClass("player")) == "DEATHKNIGHT" then
  powerPie:RegisterEvent("UNIT_RUNIC_POWER")
  powerPie:RegisterEvent("UNIT_MAXRUNIC_POWER")
else
  powerPie:RegisterEvent("UNIT_MANA")
  powerPie:RegisterEvent("UNIT_MAXMANA")
end
powerPie:SetScript("OnEvent",
  function(frame, event, ...)
    if arg1 == "player" or event == "PLAYER_ENTERING_WORLD" then
      frame:ResetPie()

      if UnitExists("player") then
        local p = (100 * UnitPower("player")) / UnitPowerMax("player")
        local c = PowerBarColor[UnitPowerType("player")]

        powerMaxValue:SetText(UnitPowerMax("player"))
        powerValue:SetText(UnitPower("player"))

        if p >= 100 then
          frame:CompletePie({c.r, c.g, c.b})
        else
          frame:AddPie(p, {c.r, c.g, c.b})
          frame:CompletePie({0, 0, 0})
        end
      else
        powerMaxValue:SetText("")
        powerValue:SetText("")
      end
    end
  end
)
